@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -204,9 +204,8 @@ Otherwise, one argument `-i' is passed to the shell."
 		  (message "Directory stack not that deep")
 		  (let ((dirstack
 			 (let ((dirstack (cons default-directory dirstack)))
-			   (append (list-tail dirstack num)
-				   (list-head dirstack
-					      (- (length dirstack) num))))))
+			   (append (drop dirstack num)
+				   (take dirstack (- (length dirstack) num))))))
 		    (set-variable! shell-dirstack (cdr dirstack))
 		    (shell-process-cd (car dirstack))))
 	      (begin
@@ -230,7 +229,7 @@ Otherwise, one argument `-i' is passed to the shell."
 	  (else
 	   (if (= num 1)
 	       (set-variable! shell-dirstack (cdr dirstack))
-	       (let ((pair (list-tail dirstack (- num 1))))
+	       (let ((pair (drop dirstack (- num 1))))
 		 (set-cdr! pair (cddr pair))))
 	   (shell-dirstack-message)))))
 
