@@ -1,8 +1,9 @@
 /* -*-C-*-
 
-$Id: uxterm.h,v 1.7 2003/02/14 18:28:24 cph Exp $
-
-Copyright (c) 1990, 1991, 1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 */
@@ -28,13 +29,11 @@ USA.
 
 #include "osterm.h"
 
-extern int EXFUN (terminal_state_buffered_p, (Ttty_state * s));
-extern void EXFUN
-  (terminal_state_buffered, (Ttty_state * s, Tchannel channel));
-extern void EXFUN
-  (terminal_state_nonbuffered, (Ttty_state * s, int fd, int polling));
-extern void EXFUN (terminal_state_raw, (Ttty_state * s, int fd));
-extern void EXFUN (get_terminal_state, (Tchannel channel, Ttty_state * s));
-extern void EXFUN (set_terminal_state, (Tchannel channel, Ttty_state * s));
+extern int terminal_state_buffered_p (Ttty_state * s);
+extern void terminal_state_buffered (Ttty_state * s, Tchannel channel);
+extern void terminal_state_nonbuffered (Ttty_state * s, int fd, int polling);
+extern void terminal_state_raw (Ttty_state * s, int fd);
+extern void get_terminal_state (Tchannel channel, Ttty_state * s);
+extern void set_terminal_state (Tchannel channel, Ttty_state * s);
 
 #endif /* SCM_UXTERM_H */
