@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -47,7 +48,7 @@ USA.
   (with-rdf-input-port port
     (lambda ()
       (post-process-parser-output
-       (parse-turtle-doc (input-port->parser-buffer port))
+       (parse-turtle-doc (textual-input-port->parser-buffer port))
        (->absolute-uri base-uri 'READ-RDF/TURTLE)))))
 
 (define parse-turtle-doc
@@ -161,8 +162,8 @@ USA.
 
 (define match:name
   (*matcher
-   (seq (alphabet alphabet:name-start-char)
-	(* (alphabet alphabet:name-char)))))
+   (seq (char-set char-set:name-start-char)
+	(* (char-set char-set:name-char)))))
 
 (define parse:prefix
   (*parser
@@ -171,8 +172,8 @@ USA.
 
 (define match:prefix-name
   (*matcher
-   (? (seq (alphabet alphabet:prefix-name-start-char)
-	   (* (alphabet alphabet:name-char))))))
+   (? (seq (char-set char-set:prefix-name-start-char)
+	   (* (char-set char-set:name-char))))))
 
 ;;;; Literals
 
@@ -201,7 +202,7 @@ USA.
 
 (define parse:language
   (*parser
-   (map utf8-string->symbol
+   (map string->symbol
 	(match (seq (+ (char-set char-set:turtle-lower))
 		    (* (seq "-"
 			    (+ (char-set char-set:turtle-lower+digit)))))))))
@@ -259,7 +260,7 @@ USA.
    (map (lambda (s) (make-rdf-literal s xsd:boolean))
 	(match (alt "true" "false")))))
 
-;;;; Alphabets
+;;;; Character sets
 
 (define char-set:turtle-hex
   (string->char-set "0123456789ABCDEF"))
@@ -277,54 +278,54 @@ USA.
   (char-set-union char-set:turtle-lower
 		  char-set:turtle-digit))
 
-(define alphabet:name-start-char
-  (scalar-values->alphabet
-   '((#x0041 . #x005A)
+(define char-set:name-start-char
+  (char-set*
+   '((#x0041 . #x005B)
      #x005F
-     (#x0061 . #x007A)
-     (#x00C0 . #x00D6)
-     (#x00D8 . #x00F6)
-     (#x00F8 . #x02FF)
-     (#x0370 . #x037D)
-     (#x037F . #x1FFF)
-     (#x200C . #x200D)
-     (#x2070 . #x218F)
-     (#x2C00 . #x2FEF)
-     (#x3001 . #xD7FF)
-     (#xF900 . #xFDCF)
-     (#xFDF0 . #xFFFD)
-     (#x10000 . #xEFFFF))))
+     (#x0061 . #x007B)
+     (#x00C0 . #x00D7)
+     (#x00D8 . #x00F7)
+     (#x00F8 . #x0300)
+     (#x0370 . #x037E)
+     (#x037F . #x2000)
+     (#x200C . #x200E)
+     (#x2070 . #x2190)
+     (#x2C00 . #x2FF0)
+     (#x3001 . #xD800)
+     (#xF900 . #xFDD0)
+     (#xFDF0 . #xFFFE)
+     (#x10000 . #xF0000))))
 
-(define alphabet:name-char
-  (alphabet+ alphabet:name-start-char
-	     (scalar-values->alphabet
-	      '(#x002D
-		(#x0030 . #x0039)
-		#x00B7
-		(#x0300 . #x036F)
-		(#x203F . #x2040)))))
+(define char-set:name-char
+  (char-set-union char-set:name-start-char
+		  (char-set*
+		   '(#x002D
+		     (#x0030 . #x003A)
+		     #x00B7
+		     (#x0300 . #x0370)
+		     (#x203F . #x2041)))))
 
-(define alphabet:prefix-name-start-char
-  (alphabet- alphabet:name-start-char (alphabet #\_)))
+(define char-set:prefix-name-start-char
+  (char-set-difference char-set:name-start-char (char-set #\_)))
 
-(define alphabet:character
-  (scalar-values->alphabet '((#x20 . #x5B) (#x5D . #x10FFFF))))
+(define char-set:character
+  (char-set* '((#x20 . #x5C) (#x5D . #x110000))))
 
-(define alphabet:ucharacter
-  (alphabet- alphabet:character (alphabet #\>)))
+(define char-set:ucharacter
+  (char-set-difference char-set:character (char-set #\>)))
 
-(define alphabet:scharacter
-  (alphabet- alphabet:character (alphabet #\")))
+(define char-set:scharacter
+  (char-set-difference char-set:character (char-set #\")))
 
-(define alphabet:lcharacter
-  (alphabet+ alphabet:character (alphabet #\tab #\newline #\return)))
+(define char-set:lcharacter
+  (char-set-union char-set:character (char-set #\tab #\newline #\return)))
 
 ;;;; Escaped strings
 
 (define (delimited-region-parser name start-delim end-delim
-				 alphabet parse-escapes)
+				 char-set parse-escapes)
   (lambda (buffer)
-    (let ((output (open-utf8-output-string))
+    (let ((output (open-output-string))
 	  (start (get-parser-buffer-pointer buffer)))
 
       (define (read-head)
@@ -334,7 +335,7 @@ USA.
 		   (copy p)
 		   (match-parser-buffer-string buffer end-delim)
 		   (finish))
-		  ((match-parser-buffer-char-in-alphabet buffer alphabet)
+		  ((match-parser-buffer-char-in-set buffer char-set)
 		   (loop))
 		  ((match-parser-buffer-char-no-advance buffer #\\)
 		   (copy p)
@@ -364,7 +365,7 @@ USA.
       (define (copy p)
 	(call-with-parser-buffer-tail buffer p
 	  (lambda (string start end)
-	    (write-substring string start end output))))
+	    (write-string string output start end))))
 
       (define (finish)
 	(vector (get-output-string output)))
@@ -409,21 +410,21 @@ USA.
   (delimited-region-parser "string"
 			   "\""
 			   "\""
-			   alphabet:scharacter
+			   char-set:scharacter
 			   parse:scharacter-escape))
 
 (define parse:long-string
   (delimited-region-parser "long string"
 			   "\"\"\""
 			   "\"\"\""
-			   alphabet:lcharacter
+			   char-set:lcharacter
 			   parse:scharacter-escape))
 
 (define parse:uriref
   (delimited-region-parser "URI reference"
 			   "<"
 			   ">"
-			   alphabet:ucharacter
+			   char-set:ucharacter
 			   parse:ucharacter-escape))
 
 ;;;; Whitespace
@@ -601,8 +602,8 @@ USA.
       (cond ((uri? o)
 	     (receive (prefix expansion)
 		 (uri->rdf-prefix o (port/rdf-prefix-registry port) #f)
-	       (if (and prefix (not (hash-table/get table prefix #f)))
-		   (hash-table/put! table prefix expansion))))
+	       (if (and prefix (not (hash-table-ref/default table prefix #f)))
+		   (hash-table-set! table prefix expansion))))
 	    ((rdf-graph? o)
 	     (check-graph o))))
 
@@ -611,10 +612,11 @@ USA.
 		(write-rdf/turtle-prefix (car p) (cdr p) port))
 	      (sort (hash-table->alist table)
 		(lambda (a b)
-		  (let ((a (symbol-name (car a)))
-			(b (symbol-name (car b))))
-		    (substring<? a 0 (fix:- (string-length a) 1)
-				 b 0 (fix:- (string-length b) 1))))))))
+		  (let ((a (symbol->string (car a)))
+			(b (symbol->string (car b))))
+		    (string<?
+		     (string-head a (fix:- (string-length a) 1))
+		     (string-head b (fix:- (string-length b) 1)))))))))
 
 (define (write-rdf/turtle-prefix prefix expansion #!optional port)
   (let ((port (if (default-object? port) (current-output-port) port)))
@@ -765,18 +767,18 @@ USA.
 		    (else #f))))
 	((rdf-bnode? o)
 	 (and (not (inline-bnode o))
-	      (call-with-utf8-output-string
+	      (call-with-output-string
 		(lambda (port)
 		  (write-rdf/nt-bnode o port)))))
 	((uri? o)
-	 (call-with-utf8-output-string
+	 (call-with-output-string
 	   (lambda (port*)
 	     (write-uri o (port/rdf-prefix-registry port) port*))))
 	((rdf-graph? o)
 	 (and (null? (rdf-graph-triples o))
 	      "{}"))
 	((rdf-literal? o)
-	 (call-with-utf8-output-string
+	 (call-with-output-string
 	   (lambda (port)
 	     (write-rdf/turtle-literal o port))))
 	(else
@@ -911,7 +913,7 @@ USA.
 
 (define (write-literal-text text port)
   (if (string-find-next-char text #\newline)
-      (let ((tport (open-utf8-input-string text)))
+      (let ((tport (open-input-string text)))
 	(write-string "\"\"\"" port)
 	(let loop ()
 	  (let ((char (read-char tport)))
@@ -936,8 +938,8 @@ USA.
 	  (let ((start (string-length expansion)))
 	    (if (*match-string match:name s start end)
 		(begin
-		  (write-string (symbol-name prefix) port)
-		  (write-substring s start end port))
+		  (write-string (symbol->string prefix) port)
+		  (write-string s port start end))
 		(write-rdf/nt-uri uri port)))
 	  (write-rdf/nt-uri uri port)))))
 
@@ -1001,7 +1003,7 @@ USA.
   (write-char #\space port))
 
 (define (write-symbol symbol port)
-  (write-string (symbol-name symbol) port))
+  (write-string (symbol->string symbol) port))
 
 (define (write-parens open close indentation port procedure)
   (write-string open port)
